@@ -1,14 +1,26 @@
 import pandas as pd
 import numpy as np
 import matplotlib.pyplot as plt
-import matplotlib.cm as gradients
 
 # ggplot stuff
 from components import colors, shapes, aes
 from components.legend import draw_legend
 from geoms import *
+from scales import *
 from utils import *
 
+import re
+
+
+def is_identity(x):
+    if x in colors.COLORS:
+        return True
+    elif x in shapes.SHAPES:
+        return True
+    elif isinstance(x, (float, int)):
+        return True
+    else:
+        return False
 
 class ggplot(object):
     """
@@ -42,24 +54,73 @@ class ggplot(object):
         self.aesthetics = aesthetics
         self.data = data
 
+        # Look for alias/lambda functions
+        # The test criteria could use some work
+        for ae, name in self.aesthetics.iteritems():
+            if name not in self.data and not is_identity(name):
+                result = re.findall(r'(?:[A-Z])|(?:[A-Za_-z0-9]+)|(?:[/*+_=\(\)-])', name)
+                if re.match("factor[(][A-Za-z_0-9]+[)]", name):
+                    m = re.search("factor[(]([A-Za-z_0-9]+)[)]", name)
+                    self.data[name] = self.data[m.group(1)].apply(str)
+                else:
+                    lambda_column = ""
+                    for item in result:
+                        if re.match("[/*+_=\(\)-]", item):
+                            pass
+                        elif re.match("^[0-9.]+$", item):
+                            pass
+                        else:
+                            item = "self.data.get('%s')" % item
+                        lambda_column += item 
+                    self.data[name] = eval(lambda_column)
         # defaults
         self.geoms= []
-        self.n_dim_x = 1
-        self.n_dim_y = 1
+        self.n_wide = 1
+        self.n_high = 1
+        self.n_dim_x = None
+        self.n_dim_y = None
         self.facets = []
+        self.facet_type = None
         # components
         self.title = None
         self.xlab = None
         self.ylab = None
+        self.xlimits = None
+        self.ylimits = None
         self.legend = {}
 
+        # continuous color configs
+        self.color_scale = None
+        self.colormap = plt.cm.Blues
+
     def __repr__(self):
         # TODO: Handle facet_wrap better so that we only have
         # as many plots as we do discrete values. Currently it
         # creates a grid of plots but doesn't use all of them
-        fig, axs = plt.subplots(self.n_dim_x, self.n_dim_y)
-        plt.subplot(self.n_dim_x, self.n_dim_y, 1)
-        
+        if self.facet_type=="grid":
+            fig, axs = plt.subplots(self.n_wide, self.n_high, 
+                    sharex=True, sharey=True)
+        elif self.facet_type=="wrap":
+            subplots_available = self.n_wide * self.n_high
+            extra_subplots = subplots_available - self.n_dim_x
+
+            fig, axs = plt.subplots(self.n_wide, self.n_high)
+            for extra_plot in axs.flatten()[-extra_subplots:]:
+                extra_plot.axis('off')
+
+            plots = [None for i in range(self.n_dim_x)]
+            for i in range(self.n_dim_x):
+                idx = (i % self.n_high) * self.n_wide + (i % self.n_wide)
+                plots[idx] = (i % self.n_wide, i % self.n_high)
+
+            plots = [plot for plot in plots if plot is not None]
+            plots = sorted(plots, key=lambda x: x[1] + x[0] * self.n_high + 1)
+
+        else:
+            fig, axs = plt.subplots(self.n_wide, self.n_high)
+
+        plt.subplot(self.n_wide, self.n_high, 1)
+
         # Faceting just means doing an additional groupby. The
         # dimensions of the plot remain the same
         if self.facets:
@@ -67,7 +128,23 @@ class ggplot(object):
             for facet, frame in self.data.groupby(self.facets):
                 for layer in self._get_layers(frame):
                     for geom in self.geoms:
-                        plt.subplot(self.n_dim_x, self.n_dim_y, cntr)
+                        if self.facet_type=="wrap":
+                            if cntr+1 > len(plots):
+                                continue
+                            pos = plots[cntr]
+                            if pos is None:
+                                continue
+                            y_i, x_i = pos
+                            pos = x_i + y_i * self.n_high + 1
+                            plt.subplot(self.n_wide, self.n_high, pos) 
+                        else:
+                            plt.subplot(self.n_wide, self.n_high, cntr)
+                            # TODO: this needs some work
+                            if (cntr % self.n_high)!=0:
+                                plt.tick_params(axis='y', which='both', 
+                                        bottom='off', top='off',
+                                        labelbottom='off')
+
                         callbacks = geom.plot_layer(layer)
                         if callbacks:
                             for callback in callbacks:
@@ -93,20 +170,25 @@ class ggplot(object):
             plt.xlabel(self.xlab)
         if self.ylab:
             plt.ylabel(self.ylab)
+        if self.xlimits:
+            plt.xlim(self.xlimits)
+        if self.ylimits:
+            plt.ylim(self.ylimits)
+
         # TODO: Having some issues here with things that shouldn't have a legend
         # or at least shouldn't get shrunk to accomodate one. Need some sort of
         # test in place to prevent this OR prevent legend getting set to True.
         if self.legend:
             if self.facets:
-                pass
-                ax_to_use = [ax for ax in axs if not isinstance(ax, np.ndarray)]
-                ax = axs[-1]
-                box = ax.get_position()
-                ax.set_position([box.x0, box.y0, box.width * 0.8, box.height])
-                cntr = 0
-                for name, legend in self.legend.iteritems():
-                    ax.add_artist(draw_legend(ax, legend, name, cntr))
-                    cntr += 1
+                if 1==2:
+                    ax_to_use = [ax for ax in axs if not isinstance(ax, np.ndarray)]
+                    ax = axs[-1]
+                    box = ax.get_position()
+                    ax.set_position([box.x0, box.y0, box.width * 0.8, box.height])
+                    cntr = 0
+                    for name, legend in self.legend.iteritems():
+                        ax.add_artist(draw_legend(ax, legend, name, cntr))
+                        cntr += 1
             else:
                 box = axs.get_position()
                 axs.set_position([box.x0, box.y0, box.width * 0.8, box.height])
@@ -136,6 +218,14 @@ class ggplot(object):
             self.ylab = self.aesthetics['y']
         mapping = mapping.dropna()
 
+        # We need to normalize size so that the points aren't really big or
+        # really small.
+        # TODO: add different types of normalization
+        if "size" in mapping:
+            mapping.size = mapping.size.astype(np.float)
+            mapping.size = 200.0 * (mapping.size - mapping.size.min()) / \
+                    (mapping.size.max() - mapping.size.min())
+
         # Here we're mapping discrete values to colors/shapes. For colors
         # we're also checking to see if we don't need to map them (i.e. if vars
         # are 'blue', 'green', etc.). The reverse mapping allows us to convert
@@ -145,9 +235,7 @@ class ggplot(object):
             possible_colors = np.unique(mapping.color)
             if set(possible_colors).issubset(set(colors.COLORS))==False:
                 if "color" in mapping._get_numeric_data().columns:
-                    # continuous
-                    # TODO: add support for more colors
-                    mapping['cmap'] = gradients.Blues
+                    mapping['cmap'] = self.colormap
                 else:
                     # discrete
                     color = colors.color_gen()
@@ -175,7 +263,10 @@ class ggplot(object):
                 frame["cmap"] = frame["cmap"][0]
                 quantiles = np.percentile(mapping.color, [0, 25, 50, 75, 100])
                 # TODO: add support for more colors
-                key_colors = ["white", "lightblue", "skyblue", "blue", "navy"]
+                if self.color_scale:
+                    key_colors = self.color_scale
+                else:
+                    key_colors = ["white", "lightblue", "skyblue", "blue", "navy"]
                 legend["color"] = dict(zip(key_colors, quantiles))
             layers.append(frame)
         else:
@@ -190,10 +281,6 @@ class ggplot(object):
                         else:
                             aes_name = name
                         if ae=="color":
-                            # TODO: Handle discrete vs. continuous colors here
-                            # import matplotlib as mpl 
-                            # cmap=mpl.cm.Blues
-                            # plt.scatter(x, y, c=y, s=500, cmap=mpl.cm.gray)
                             label = rev_color_mapping.get(aes_name, aes_name)
                             legend[ae][frame[ae]] = label
                         elif ae=="shape" or ae=="marker":
